@@ -116,7 +116,7 @@ pub struct ParsedArguments {
     /// arguments are incompatible with rewrite_includes_only
     pub suppress_rewrite_includes_only: bool,
     /// Arguments are incompatible with preprocessor cache mode
-    pub too_hard_for_preprocessor_cache_mode: bool,
+    pub too_hard_for_preprocessor_cache_mode: Option<OsString>,
 }
 
 impl ParsedArguments {
@@ -275,7 +275,7 @@ where
                     .map(|f| f.path())
                     .collect()
             })
-            .unwrap_or(Vec::default())
+            .unwrap_or_default()
     }
 }
 
@@ -382,7 +382,19 @@ where
         // Try to look for a cached preprocessing step for this compilation
         // request.
         let preprocessor_cache_mode_config = storage.preprocessor_cache_mode_config();
-        let mut preprocessor_key = if preprocessor_cache_mode_config.use_preprocessor_cache_mode {
+        let too_hard_for_preprocessor_cache_mode =
+            parsed_args.too_hard_for_preprocessor_cache_mode.is_some();
+        if let Some(arg) = &parsed_args.too_hard_for_preprocessor_cache_mode {
+            debug!(
+                "parse_arguments: Cannot use preprocessor cache because of {:?}",
+                arg
+            );
+        }
+        // Disable preprocessor cache when doing distributed compilation
+        let mut preprocessor_key = if !may_dist
+            && preprocessor_cache_mode_config.use_preprocessor_cache_mode
+            && !too_hard_for_preprocessor_cache_mode
+        {
             preprocessor_cache_entry_hash_key(
                 &executable_digest,
                 parsed_args.language,
@@ -398,8 +410,9 @@ where
         };
         if let Some(preprocessor_key) = &preprocessor_key {
             if cache_control == CacheControl::Default {
-                if let Some(mut seekable) =
-                    storage.get_preprocessor_cache_entry(preprocessor_key)?
+                if let Some(mut seekable) = storage
+                    .get_preprocessor_cache_entry(preprocessor_key)
+                    .await?
                 {
                     let mut buf = vec![];
                     seekable.read_to_end(&mut buf)?;
@@ -416,10 +429,13 @@ where
                             "Preprocessor cache updated because of time macros: {preprocessor_key}"
                         );
 
-                        if let Err(e) = storage.put_preprocessor_cache_entry(
-                            preprocessor_key,
-                            preprocessor_cache_entry,
-                        ) {
+                        if let Err(e) = storage
+                            .put_preprocessor_cache_entry(
+                                preprocessor_key,
+                                preprocessor_cache_entry,
+                            )
+                            .await
+                        {
                             debug!("Failed to update preprocessor cache: {}", e);
                             update_failed = true;
                         }
@@ -483,16 +499,14 @@ where
             debug!("removing files {:?}", &outputs);
 
             let v: std::result::Result<(), std::io::Error> =
-                outputs.values().fold(Ok(()), |r, output| {
-                    r.and_then(|_| {
-                        let mut path = args_cwd.clone();
-                        path.push(&output.path);
-                        match fs::metadata(&path) {
-                            // File exists, remove it.
-                            Ok(_) => fs::remove_file(&path),
-                            _ => Ok(()),
-                        }
-                    })
+                outputs.values().try_for_each(|output| {
+                    let mut path = args_cwd.clone();
+                    path.push(&output.path);
+                    match fs::metadata(&path) {
+                        // File exists, remove it.
+                        Ok(_) => fs::remove_file(&path),
+                        _ => Ok(()),
+                    }
                 });
             if v.is_err() {
                 warn!("Could not remove files after preprocessing failed!");
@@ -570,6 +584,7 @@ where
 
                 if let Err(e) = storage
                     .put_preprocessor_cache_entry(&preprocessor_key, preprocessor_cache_entry)
+                    .await
                 {
                     debug!("Failed to update preprocessor cache: {}", e);
                 }
@@ -622,7 +637,7 @@ const INCBIN_DIRECTIVE: &[u8] = b".incbin";
 fn process_preprocessed_file(
     input_file: &Path,
     cwd: &Path,
-    bytes: &mut Vec<u8>,
+    bytes: &mut [u8],
     included_files: &mut HashMap<PathBuf, String>,
     config: PreprocessorCacheModeConfig,
     time_of_compilation: std::time::SystemTime,
